#!/usr/bin/env python
'''
    Our unit tests for the updateinfo master object
'''
from __future__ import print_function

import copy
import os
import sys
import unittest

from updateinfo import Updateinfo

class UpdateinfoTestCase(unittest.TestCase):
    '''
        Test the top level object, if it doesn't work we've a real problem
    '''

    def test_read_existing_file(self):
        '''
            Read in the sample updateinfo.xml files
        '''
        for filename in ('docs/samples/EPEL6-updateinfo.xml', 'docs/samples/Fedora-updateinfo.xml', 'docs/samples/simple.xml'):
            testupdateinfo = Updateinfo()
            self.assertTrue(testupdateinfo.from_xml_file(filename), msg="Unable to read " + filename)

    def test_merge_with_file(self):
        '''
            Try merging the sample updateinfo files
        '''
        testupdateinfo = Updateinfo()
        for filename in ('docs/samples/EPEL6-updateinfo.xml', 'docs/samples/Fedora-updateinfo.xml', 'docs/samples/simple.xml'):
            self.assertTrue(testupdateinfo.merge_with_file(filename), msg="Unable to merge " + filename)


    def test_xml_validates(self):
        '''
            Try validating the merged the sample updateinfo files
        '''
        testupdateinfo = Updateinfo()
        for filename in ('docs/samples/EPEL6-updateinfo.xml', 'docs/samples/Fedora-updateinfo.xml', 'docs/samples/simple.xml'):
            testupdateinfo.merge_with_file(filename)
        xmltxt = testupdateinfo.get_xml(header=False, validate=False)
        self.assertTrue(testupdateinfo.validate_xml_text(xmltxt), msg="Couldn't validate output")

    def test_obj_validates(self):
        '''
            Try validating the merged the sample updateinfo files
        '''
        testupdateinfo = Updateinfo()
        for filename in ('docs/samples/EPEL6-updateinfo.xml', 'docs/samples/Fedora-updateinfo.xml', 'docs/samples/simple.xml'):
            testupdateinfo.merge_with_file(filename)
        self.assertTrue(testupdateinfo.validate_xml_text(), msg="Couldn't validate self output")

    def test_bad_xml_no_valid(self):
        '''
            Try validating some bad xml
        '''
        import lxml
        testupdateinfo = Updateinfo()
        bad_text = '<updates><update><tag/></update></updates>'
        expected = False
        try:
            testupdateinfo.validate_xml_text(bad_text)
        except lxml.etree.XMLSyntaxError:
            expected = True
        except DocumentInvalid:
            expected = True
        self.assertTrue(expected, msg="I validated bad_xml")

    def test_len(self):
        '''
            Make sure len works
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        self.assertEqual(len(testupdateinfo), 2, msg="Bad length")

    def test_getitem(self):
        '''
            try fetch via testupdateinfo[id]
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        thisid = testupdateinfo['SLID-11111']
        thisid_type = unicode(type(thisid))
        expected_type = u"<class 'updateinfo.entry.Entry'>"
        self.assertEqual(thisid_type, expected_type, msg="Unexpected type: " + thisid_type)

    def test_setitem(self):
        '''
            set like a dict
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        preadd = len(testupdateinfo)

        thisid = copy.deepcopy(testupdateinfo['SLID-11111'])
        thisid.set_id('testid-1234')
        testupdateinfo['testid-1234'] = thisid

        postadd = len(testupdateinfo)
        result = postadd - preadd
        self.assertEqual(result, 1, msg="Couldn't add like a dict")

    def test_setitem_is_sane(self):
        '''
            Make sure set item sets sanely
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        preadd = len(testupdateinfo)

        thisid = copy.deepcopy(testupdateinfo['SLID-11111'])
        thisid.set_id('testid-1234')
        try:
            testupdateinfo['testid-4321'] = thisid
        except KeyError:
            pass

        postadd = len(testupdateinfo)
        result = postadd - preadd
        self.assertEqual(result, 0, msg="Couldn't add bad key ids")

    def test_delitem(self):
        '''
            del like a dict
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        predel = len(testupdateinfo)

        del testupdateinfo['SLID-11111']

        postdel = len(testupdateinfo)
        result = predel - postdel
        self.assertEqual(result, 1, msg="Couldn't del like a dict")

    def test_iter(self):
        '''
            Make sure for i in x: works
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        count = 0
        for entry in testupdateinfo:
            count = count + 1
        self.assertEqual(len(testupdateinfo), 2, msg="Couldn't loop")

    def test_contains(self):
        '''
            Make sure if x in obj works
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        found = False
        if 'SLID-11111' in testupdateinfo:
            found = True
        self.assertTrue(found, msg="Couldn't do if x in obj")

    def test_as_str(self):
        '''
            Make sure stringy bits work
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        found = False
        xmltxt = str(testupdateinfo)
        if 'SLID-11111' in xmltxt:
            found = True
        self.assertTrue(found, msg="Couldn't str(obj)")

    def test_has_key(self):
        '''
            Make sure obj.has_key works
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        found = False
        if testupdateinfo.has_key('SLID-11111'):
            found = True
        self.assertTrue(found, msg="Couldn't do obj.has_key()")

    def test_keys(self):
        '''
            Make sure obj.keys works
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        expect = ('SLID-11111', 'SLID-2345')
        self.assertEqual(tuple(testupdateinfo.keys()), expect, msg="incorrect keys list")

    def test_force_updatefrom(self):
        '''
           Does force_updatefrom work
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        testupdateinfo.force_updatefrom('IMATESTIMATESTIMATEST')

        xmltxt = testupdateinfo.get_xml(header=False, validate=False)

        found = False
        if '<update type="enhancement" from="IMATESTIMATESTIMATEST" status="stable" version="1.4">' in xmltxt:
            found = True

        self.assertTrue(found, msg="Couldn't obj.force_updatefrom()")

    def test_force_status(self):
        '''
           Does force_status work
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        testupdateinfo.force_status('IMATESTIMATESTIMATEST')

        xmltxt = testupdateinfo.get_xml(header=False, validate=False)

        found = False
        if '<update type="enhancement" from="asdf@jkl.com" status="IMATESTIMATESTIMATEST" version="1.4">' in xmltxt:
            found = True

        self.assertTrue(found, msg="Couldn't obj.force_status()")

    def test_force_release_name(self):
        '''
           Does force_release_name work
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        testupdateinfo.force_release_name('IMATESTIMATESTIMATEST')
        
        xmltxt = testupdateinfo.get_xml(header=False, validate=False)

        found = False
        if '<release>IMATESTIMATESTIMATEST</release>' in xmltxt:
            found = True

        self.assertTrue(found, msg="Couldn't obj.force_release_name()")

    def test_force_collection_name_noshort(self):
        '''
           Does force_collection_name work
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        testupdateinfo.force_collection_name('IMATESTIMATESTIMATEST')

        xmltxt = testupdateinfo.get_xml(header=False, validate=False)

        found = False
        if '<name>IMATESTIMATESTIMATEST</name>' in xmltxt:
            found = True

        self.assertTrue(found, msg="Couldn't obj.force_collection_name() noshort")

    def test_force_collection_name(self):
        '''
           Does force_collection_name work
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        testupdateinfo.force_collection_name('IMATESTIMATESTIMATEST', 'TSETAMITSETAMITSETAMI')

        xmltxt = testupdateinfo.get_xml(header=False, validate=False)

        found = False
        if '<name>IMATESTIMATESTIMATEST</name>' in xmltxt:
            if 'short="TSETAMITSETAMITSETAMI">' in xmltxt:
                found = True

        self.assertTrue(found, msg="Couldn't obj.force_collection_name()")

    def test_set_xsl_url(self):
        '''
            Can I set the xsl url
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        testupdateinfo.set_xsl_url('IMATESTIMATESTIMATEST')

        xmltxt = testupdateinfo.get_xml(header=False, validate=False)
        found = False
        if '<?xml-stylesheet type="text/xsl" href="IMATESTIMATESTIMATEST"?>' in xmltxt:
            found = True

        self.assertTrue(found, msg="Couldn't set xsl_url")

    def test_set_xsd_url(self):
        '''
            Can I set the xsd url
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        testupdateinfo.set_xsd_url('IMATESTIMATESTIMATEST')

        xmltxt = testupdateinfo.get_xml(header=False, validate=False)
        found = False
        if '<updates xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="IMATESTIMATESTIMATEST">' in xmltxt: 
            found = True

        self.assertTrue(found, msg="Couldn't set xsd_url")

    def test_has_ref_url(self):
        '''
            Check that we can look for ref urls
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.merge_with_file('docs/samples/simple.xml')
        self.assertTrue(testupdateinfo.has_ref_url('http://mybugzillaurl.com/bug#1234'), msg="Can't find by ref_url")

    def test_has_filename(self):
        '''
            Check that we can look for files
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.merge_with_file('docs/samples/simple.xml')
        self.assertTrue(testupdateinfo.has_filename('python-Updateinfo-0.1.5-0.sl6.noarch.rpm'), msg="Can't find by filename")

    def test_has_entry(self):
        '''
            does has_entry work
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        self.assertTrue(testupdateinfo.has_entry('SLID-11111'), msg="Couldn't obj.has_entry")

    def test_add_entry(self):
        '''
            does add_entry work
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        preadd = len(testupdateinfo)

        thisid = copy.deepcopy(testupdateinfo['SLID-11111'])
        thisid.set_id('testid-1234')
        testupdateinfo.add_entry(thisid)

        postadd = len(testupdateinfo)
        result = postadd - preadd
        self.assertEqual(result, 1, msg="Couldn't obj.add_entry")

    def test_del_entry(self):
        '''
            does del_entry work
        '''
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        predel = len(testupdateinfo)

        testupdateinfo.del_entry('SLID-11111')

        postdel = len(testupdateinfo)
        result = predel - postdel
        self.assertEqual(result, 1, msg="Couldn't obj.del_entry")

    def test_print_whole(self):
        '''
            Check that we can print(thing)
        '''
        f = open(os.devnull, 'w')
        sys.stdout = f
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        canprint = False
        try:
            print(testupdateinfo)
            canprint = True
        except:
            pass
        self.assertTrue(canprint, msg="Couldn't print the whole object")

    def test_print_update(self):
        '''
            Check that we can print(thing['id'])
        '''
        f = open(os.devnull, 'w')
        sys.stdout = f
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        canprint = False
        try:
            for id in testupdateinfo:
                print(testupdateinfo[id])
            canprint = True
        except:
            pass
        self.assertTrue(canprint, msg="Couldn't print the individual updates")

    def test_print_update_collections(self):
        '''
            Check that we can print(thing['id']['collection'])
        '''
        f = open(os.devnull, 'w')
        sys.stdout = f
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        canprint = False
        try:
            for id in testupdateinfo:
                for collection in testupdateinfo[id]:
                    print(testupdateinfo[id][collection])
            canprint = True
        except:
            pass
        self.assertTrue(canprint, msg="Couldn't print the individual update collections")

    def test_print_update_collection_packages(self):
        '''
            Check that we can print(thing['id']['collection']['pkg'])
        ''' 
        f = open(os.devnull, 'w')
        sys.stdout = f
        testupdateinfo = Updateinfo()
        testupdateinfo.from_xml_file('docs/samples/simple.xml')
        canprint = False
        try:
            for id in testupdateinfo:
                for collection in testupdateinfo[id]:
                    for pkg in testupdateinfo[id][collection]:
                        print(testupdateinfo[id][collection][pkg])
            canprint = True
        except:
            pass
        self.assertTrue(canprint, msg="Couldn't print the individual update collection packages")


if __name__ == '__main__':
    unittest.main()
